# import os
# from flask_sqlalchemy import SQLAlchemy
# from flask import Flask
#
# # initialize Flask app
# app = Flask(__name__)
# app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///database/app.db'
# app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
#
# db = SQLAlchemy(app)
#
# # 用户表
# class User(db.Model):
#     id = db.Column(db.Integer, primary_key=True)
#
#
# # 用户资料表
# class UserProfile(db.Model):
#     id = db.Column(db.Integer, primary_key=True)
#
#
# # 产品表
# class Product(db.Model):
#     id = db.Column(db.Integer, primary_key=True)
#
#
# # 订单表
# class Order(db.Model):
#     id = db.Column(db.Integer, primary_key=True)
#
#
# # 订单详情表
# class OrderItem(db.Model):
#     id = db.Column(db.Integer, primary_key=True)
#
#
# # 论坛帖子表
# class ForumPost(db.Model):
#     id = db.Column(db.Integer, primary_key=True)
#
#
# # 论坛评论表
# class ForumComment(db.Model):
#     id = db.Column(db.Integer, primary_key=True)
#
#
# # create database and tables
# if __name__ == '__main__':
#     os.makedirs("database", exist_ok=True)
#     db.create_all()
#     print("Database and tables have been created.")


import os
from flask_sqlalchemy import SQLAlchemy
from flask import Flask
from datetime import datetime
from pathlib import Path

from sqlalchemy import inspect


BASE_DIR = Path(__file__).parent

app = Flask(__name__)

app.config['SQLALCHEMY_DATABASE_URI'] = f'sqlite:///{BASE_DIR}/database/app.db'
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
db = SQLAlchemy(app)


# 用户表
class User(db.Model):
    __tablename__ = 'users'
    user_id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    username = db.Column(db.String(50), unique=True, nullable=False)
    email = db.Column(db.String(100), unique=True, nullable=False)
    password_hash = db.Column(db.String(255), nullable=False)
    role = db.Column(db.String(10), nullable=False, default='user')
    created_at = db.Column(db.DateTime, default=datetime.utcnow)

    # 关系
    health_profile = db.relationship('UserHealthProfile', backref='user', uselist=False)
    recommendations = db.relationship('Recommendation', backref='user')
    cart_items = db.relationship('ShoppingCart', backref='user')
    orders = db.relationship('Order', backref='user')
    posts = db.relationship('ForumPost', backref='author')
    likes = db.relationship('PostLike', backref='user')
    comments = db.relationship('PostComment', backref='author')


# 用户健康信息表
class UserHealthProfile(db.Model):
    __tablename__ = 'user_health_profiles'
    profile_id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    user_id = db.Column(db.Integer, db.ForeignKey('users.user_id'))
    gender = db.Column(db.String(10))
    birth_date = db.Column(db.Date)
    height = db.Column(db.Numeric(5, 2))
    weight = db.Column(db.Numeric(5, 2))
    activity_level = db.Column(db.String(20))
    health_goal = db.Column(db.Boolean)


# 系统建议表
class Recommendation(db.Model):
    __tablename__ = 'recommendations'
    recommendation_id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    user_id = db.Column(db.Integer, db.ForeignKey('users.user_id'))
    recommended_fat = db.Column(db.Float)
    recommended_carbohydrates = db.Column(db.Float)
    recommended_protein = db.Column(db.Float)
    daily_calorie_needs = db.Column(db.Numeric(10, 2))


# 产品表
class Product(db.Model):
    __tablename__ = 'products'
    product_id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    name = db.Column(db.String(100), nullable=False)
    description = db.Column(db.Text)
    price = db.Column(db.Numeric(10, 2), nullable=False)
    image_url = db.Column(db.String(255))
    inventory = db.Column(db.Integer, default=0)

    # 关系
    nutrition = db.relationship('ProductNutrition', backref='product', uselist=False)
    recipes = db.relationship('ProductRecipe', backref='product')
    cart_items = db.relationship('ShoppingCart', backref='product')
    orders = db.relationship('Order', backref='product')


# 产品营养表
class ProductNutrition(db.Model):
    __tablename__ = 'product_nutrition'
    nutrition_id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    product_id = db.Column(db.Integer, db.ForeignKey('products.product_id'))
    fat_per_100g = db.Column(db.Numeric(5, 2))
    protein_per_100g = db.Column(db.Numeric(5, 2))
    carbohydrates_per_100g = db.Column(db.Numeric(5, 2))
    energy_kcal = db.Column(db.Numeric(6, 2))
    sodium_mg = db.Column(db.Numeric(6, 2), default=0)
    fiber_g = db.Column(db.Numeric(5, 2), default=0)


# 购物车表
class ShoppingCart(db.Model):
    __tablename__ = 'shopping_carts'
    user_id = db.Column(db.Integer, db.ForeignKey('users.user_id'), primary_key=True)
    product_id = db.Column(db.Integer, db.ForeignKey('products.product_id'), primary_key=True)
    quantity = db.Column(db.Integer, default=1)


# 原材料表
class RawMaterial(db.Model):
    __tablename__ = 'raw_materials'
    material_id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    name = db.Column(db.String(100), nullable=False)
    current_stock = db.Column(db.Numeric(10, 2), default=0)
    min_stock_level = db.Column(db.Numeric(10, 2))
    unit = db.Column(db.String(10))
    expiration_days = db.Column(db.Integer)

    # 关系
    recipes = db.relationship('ProductRecipe', backref='material')


# 产品配方表
class ProductRecipe(db.Model):
    __tablename__ = 'product_recipes'
    product_id = db.Column(db.Integer, db.ForeignKey('products.product_id'), primary_key=True)
    material_id = db.Column(db.Integer, db.ForeignKey('raw_materials.material_id'), primary_key=True)
    quantity = db.Column(db.Numeric(10, 2), nullable=False)


# 订单表
class Order(db.Model):
    __tablename__ = 'orders'
    order_id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    user_id = db.Column(db.Integer, db.ForeignKey('users.user_id'))
    product_id = db.Column(db.Integer, db.ForeignKey('products.product_id'))
    quantity = db.Column(db.Integer, nullable=False)
    status = db.Column(db.Boolean, default=False)
    total_amount = db.Column(db.Numeric(10, 2), nullable=False)
    created_at = db.Column(db.DateTime, default=datetime.utcnow)
    updated_at = db.Column(db.DateTime, default=datetime.utcnow, onupdate=datetime.utcnow)


# 论坛帖子表
class ForumPost(db.Model):
    __tablename__ = 'forum_posts'
    post_id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    user_id = db.Column(db.Integer, db.ForeignKey('users.user_id'))
    title = db.Column(db.String(100), nullable=False)
    content = db.Column(db.Text, nullable=False)
    is_sticky = db.Column(db.Boolean, default=False)
    created_at = db.Column(db.DateTime, default=datetime.utcnow)
    updated_at = db.Column(db.DateTime, default=datetime.utcnow, onupdate=datetime.utcnow)

    # 关系
    likes = db.relationship('PostLike', backref='post')
    comments = db.relationship('PostComment', backref='post')


# 帖子点赞表
class PostLike(db.Model):
    __tablename__ = 'post_likes'
    user_id = db.Column(db.Integer, db.ForeignKey('users.user_id'), primary_key=True)
    post_id = db.Column(db.Integer, db.ForeignKey('forum_posts.post_id'), primary_key=True)
    created_at = db.Column(db.DateTime, default=datetime.utcnow)


# 帖子评论表
class PostComment(db.Model):
    __tablename__ = 'post_comments'
    comment_id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    user_id = db.Column(db.Integer, db.ForeignKey('users.user_id'))
    post_id = db.Column(db.Integer, db.ForeignKey('forum_posts.post_id'))
    content = db.Column(db.Text, nullable=False)
    created_at = db.Column(db.DateTime, default=datetime.utcnow)


if __name__ == '__main__':

    db_dir = BASE_DIR / "database"
    db_dir.mkdir(exist_ok=True, parents=True)
    try:
        with app.app_context():
            db.create_all()
            print("所有表已创建！")
            inspector = inspect(db.engine)
            print("已创建的表:", inspector.get_table_names())
            print(f"数据库已成功生成在: {db_dir / 'app.db'}")
    except Exception as e:
        print(f"创建数据库失败: {str(e)}")